module.exports = function(gulp, plugins, globs, project_info, scripts_entry, views_entry, browserSync) {
    'use strict';

    var release = process.argv.indexOf('--release') !== -1,
        path = require('path'),
        named = require('vinyl-named'),
        webpack = require('webpack'),
        webpack_stream = require('webpack-stream'),
        run_sequence = require('run-sequence'),
        WebpackPathOrderPlugin = require('path-order-webpack-plugin'),
        postcssImport = require('postcss-import'),
        DEBUG = release,
        VERBOSE = release,
        AUTOPREFIXER_BROWSERS = [
            'Android 2.3',
            'Android >= 4',
            'Chrome >= 35',
            'Firefox >= 31',
            'Explorer >= 9',
            'iOS >= 7',
            'Opera >= 12',
            'Safari >= 7.1',
        ],
        webpack_config_builder = require('webpack-config')
        .fromObject,
        webpack_config = function(release) {
            var root = path.join(process.cwd(), project_info.destinations.base_dir);

            var r = {
                entry: scripts_entry,
                stats: {
                    colors: true,
                    reasons: DEBUG,
                    hash: VERBOSE,
                    version: VERBOSE,
                    timings: true,
                    chunks: VERBOSE,
                    chunkModules: VERBOSE,
                    cached: VERBOSE,
                    cachedAssets: VERBOSE,
                },
                output: {
                    filename: '[name].js'
                },
                resolve: {
                    root: [
                        root,
                        path.join(process.cwd(), 'app'),
                        path.join(process.cwd(), 'bower_components')
                    ],
                    modulesDirectories: [
                        'bower_components', 'node_modules', 'scripts', 'libs', 'widgets', 'views', 'pages', 'vendor'
                    ]
                },
                plugins: null,
                module: {
                    loaders: [{
                        test: /\.jade$/,
                        loader: 'jade'
                    }, {
                        test: /\.json$/,
                        loader: 'json'
                    }, {
                        test: /(\.yaml|\.yml)$/,
                        loader: 'yml!json'
                    }, {
                        test: /\.scss$/,
                        loader: 'style/useable!css!postcss!sass'
                    }, {
                        test: /\.css$/,
                        loader: 'style!css!postcss'
                    }]
                },
                postcss: function plugins() {
                    var self = this;
                    return [
                        postcssImport({
                            onImport: function(files) {
                                files.forEach(self.addDependency);
                            },
                        }),
                        require('postcss-nested')(),
                        require('postcss-cssnext')({
                            autoprefixer: AUTOPREFIXER_BROWSERS
                        }),
                    ];
                },
            };

            if (!release) {
                r.devtool = 'source-map';
                r.output.sourceMapFilename = '[name].map';
                r.plugins = [];
            } else {
                r.plugins = [
                    // include only cz lang pack for moment.js
                    new webpack.ContextReplacementPlugin(/moment[\\\/]locale$/, /^\.\/cs$/),
                    //minify and mangle resulting build
                    new webpack.optimize.UglifyJsPlugin({
                        minimize: true,
                        compress: {
                            conditionals: false,
                            drop_console: true
                        },
                        comments: false,
                        join_vars: true,
                        sequences: true,
                        mangle: {
                            except: [
                                '$super', '$', 'jQuery', 'exports', 'require', 'window', 'FEDS'
                            ]
                        }
                    })
                ];
            }

            // include everywhere plugins
            r.plugins.push(new webpack.ProvidePlugin({
                jQuery: 'expose?jQuery!jquery',
                $: 'expose?$!jquery',
            }));

            // resolver for bower
            r.plugins.push(new webpack.ResolverPlugin(
                new webpack.ResolverPlugin.DirectoryDescriptionFilePlugin('bower.json', ['main'])
            ));

            // code splitting
            r.plugins.push(new webpack.optimize.CommonsChunkPlugin({
                name: 'commons',
                filename: 'commons.js',
                minChunks: 2
            }));
            r.plugins.push(new webpack.dependencies.LabeledModulesPlugin());
            r.plugins.push(new webpack.optimize.OccurenceOrderPlugin());
            r.plugins.push(new webpack.optimize.DedupePlugin());
            r.plugins.push(new WebpackPathOrderPlugin());
            r.plugins.push(new webpack.IgnorePlugin(/(?:(?!components\/).).+\.(md|scss)/));
            r.plugins.push(new webpack.optimize.DedupePlugin());
            return webpack_config_builder(r);
        };

    return {
        injectEntryPoints: function() {
            var fragments_to_inject = gulp.src(globs.scripts.compile, {
                read: false
            });
            return gulp.src(path.join(process.cwd(), 'feds', 'config.webpack_entry.js'))
                .pipe(plugins.inject(fragments_to_inject, {
                    starttag: '// inject:entry_points',
                    endtag: '// endinject',
                    relative: true,
                    transform: function(filepath) {
                        var name = path.basename(filepath).replace(/\.[a-z]{1,4}$/gi, '');
                        var scrPath = filepath;
                        scrPath = filepath.replace(/^\.{0,2}\/((app|scripts|pages|views|elements|widgets)\/)+/gi, '');
                        scrPath = scrPath.replace(new RegExp(name + '\/', 'gi'), '').replace('/', '-').replace('\\', '-').replace(/\.[a-z]{1,4}$/gi, '');
                        return '\'' + scrPath + '\': [\'./' + path.relative('../', filepath) + '\'],';
                    }
                }))
                .pipe(gulp.dest(path.join(process.cwd(), './feds')));
        },
        webpack: function() {
            return gulp.src(globs.scripts.compile)
                .pipe(plugins.plumber())
                .pipe(plugins.if(!release, plugins.sourcemaps.init()))
                .on('error', function(error) {
                    plugins.util.beep();
                    var stdout = plugins.util.colors.red('\nError ocurred during JS compilation.\n') + error;
                    plugins.util.log(stdout);
                    return stdout;
                })
                .pipe(named())
                .pipe(webpack_stream(webpack_config(release)))
                .pipe(plugins.if(!release, plugins.sourcemaps.write()))
                .pipe(gulp.dest(globs.scripts.dest));

        },
        injectBlocks: function() {
            var fragmentsToInject = gulp.src(globs.scripts.inject.what, {
                read: false
            });

            return gulp.src(path.join(globs.scripts.inject.where, 'autoinit.js'))
                .pipe(plugins.inject(fragmentsToInject, {
                    starttag: '// inject:blocks',
                    endtag: '// endinject',
                    relative: true,
                    transform: function(filepath) {
                        return '\'' + path.basename(filepath)
                            .replace(/.js/i, '') + '\': require(\'' + filepath + '\'),';
                    }
                }))
                .pipe(gulp.dest(globs.scripts.inject.where));
        }
    };

};
